/*
 * alibavaClient.h
 *
 *  Created on: Jan 21, 2014
 *      Author: lacasta
 */

#ifndef ALIBAVACLIENT_H_
#define ALIBAVACLIENT_H_
#include <iosfwd>
#include <string>
#include <vector>
#include <soapalibavaSOAPProxy.h>
#include <Histogram.h>

/*
 * This is a class that handles the communication with the alibava-gui
 * SOAP server
 */
class Alibava
{
    public:
        typedef std::vector<Histogram *> HstList;
    private:
        alibavaSOAPProxy *server;
        HstList histograms;

        void reset_histograms();
        void read_histograms(std::istream &is);
    public:
        Alibava(const char *uri=0);
        ~Alibava();

        void connect(const char *uri);
        int getStatus(ns1__Status &status);
        int setDataFile(const char *ofile);
        int setParameter(const std::string &name, const std::string &value);
        int stopRun();
        int getMask();
        int Reset();
        int startRun(ns1__RunType type, bool async, int nevt, int nbin, int xmin, int xmax, int nsample);
        int startPedestalRun(int nevt, bool async, int nbin=512, int xmin=-512, int xmax=512, int nsample=100);
        int startSourceRun(int nevt, bool async, int nbin=512, int xmin=-512, int xmax=512, int nsample=100);
        int startLaserRun(int nevt, bool async, int nbin=512, int xmin=-512, int xmax=512, int nsample=100);
        int startCalibrationRun(bool async, int nevt_per_point=50, int npts=10, double vfrom=0.0, double vto=30000, bool is_charge=true);
        int startChargeScan(bool async, int nevt_per_point=50, int npts=10, double vfrom=0.0, double vto=30000);
        int startLaserSync(bool async, int nevt_per_point=50, int npts=10, double vfrom=0.0, double vto=512);
        int getRunData();
        Histogram *get_histogram(int ix) { return histograms[ix]; }

};

std::ostream &operator<<(std::ostream &os, const ns1__Status &st);
#endif /* ALIBAVACLIENT_H_ */
