/* -*- mode: c++ -*- */
#ifndef __ConfigFile_h__
#define __ConfigFile_h__

#include <string>
#include <KeyFile.h>
#include <signal_emitter.h>

class ConfigFile
{
    private:
        KeyFile *keyfile;
        std::string group;
        Emitter *emitter;

        void cpy(const ConfigFile &F);

        int _firmware;

    public:
        ConfigFile(std::string g="<none>", KeyFile *f=0);
        ConfigFile(const ConfigFile &F);
        ConfigFile &operator=(const ConfigFile &F);
        virtual ~ConfigFile();

        int firmware() const { return _firmware; }
        void firmware(int fw) { _firmware = fw; }

        KeyFile *key_file() { return keyfile; }
        const KeyFile *key_file() const { return keyfile; }
        void set_file(KeyFile *f);
        void set_group(const std::string &s)
        {
            group = s;
        }
        std::string get_group() const
        {
            return group;
        }

        bool exists() const;

        std::string get_string(const char *key);
        void set_string(const char *key, const std::string &value);

        bool get_bool(const char *key, bool default_value=false);
        void set_bool(const char *key, bool v);

        int get_int(const char *key, int default_value=0);
        void set_int(const char *key, int value);

        double get_double(const char *, double default_value=0.0);
        void set_double(const char *, double value);

        double *get_double_list(const char *, int &length);
        void set_double_list(const char *, double *, int length);

        int *get_int_list(const char *, int &length);
        void set_int_list(const char *, int *, int length);


        virtual std::string get_data()
        {
            std::string out;
            return out;
        }
        virtual void load_data()
        {
        }
        virtual void save_data()
        {
        }


        /*
         * Connects to notify changes in parameters.
         * The function isgnature is
         * func(GObject *, gchar *group_name, gchar *key, gpointer data)
         * Gobject can be ignored
         */
        int connect(GCallback func, gpointer data);

        friend std::ostream &operator<<(std::ostream &os, const ConfigFile &);
};
std::ostream &operator<<(std::ostream &os, const ConfigFile &);
#endif
