/*
 * DAQmanager.h
 *
 *  Created on: 29 Jan 2011
 *      Author: lacasta
 */

#ifndef DAQMANAGER_H_
#define DAQMANAGER_H_

#include <string>
#include <RunManager.h>
#include <Alibava.h>
#include <DataFile.h>
#include <DataMonitor.h>
#include <KeyFile.h>

struct DAQmanagerPriv;

class DAQmanager : public DataReceiver
{
    private:
        int nevts;
        int received;
        int expected;
        Alibava *board;
        DataFile *ofile;
        DataFile::FileType format;
        DataMonitor *monitor;
        RunManager::RunType run_type;
        RunManager *rm;

        bool _starting;
        bool _running;
        bool _stop_running;
        bool _is_gui;

        // Config file
        KeyFile *config_file;

        DAQmanagerPriv *_priv;

        void running(bool x);

        /** TODO:
         * Implement DAtaMOnitor in RunManager instead of AlibabaGUI
         */
        static DAQmanager *_mngr;

    protected:
        void set_as_gui(bool val) { _is_gui=val; }

    public:
        DAQmanager(int nn, Alibava *B, RunManager::RunType tp = RunManager::Unknown);
        virtual ~DAQmanager();

        void set_expected(int x) { expected = nevts = x; }
        void set_received(int x) { received = x; }
        int get_expected() const { return expected; }
        int get_received() const { return received; }
        double get_fraction_received() const { return expected > 0.0 ? double(received)/double(expected) : 0.0; }

        void set_monitor(DataMonitor *m) { monitor = m; }
        DataMonitor *get_monitor() { return monitor; }

        void set_data_file(const std::string &fnam);
        void set_run_type( RunManager::RunType tp);
        RunManager::RunType get_run_type() { return run_type; }
        RunManager *get_run_manager() { return rm; }

        DataFile *out_file() { return ofile; }
        void flush_data_file();

        virtual int open(const char *fnam);
        virtual int reset();
        virtual int prepare_for_run();
        virtual int acquire();
        virtual int stop_run();
        virtual void device_opened() {}
        virtual void receive (const AlibavaData *);
        virtual void config_loaded();
        virtual void process_event(const AlibavaData *);
        virtual void write_event(const AlibavaData *);
        virtual void acquire_thread(bool is_thread=false) {}

        Alibava *get_board() { return board; }


        DataFile::FileType file_format() const { return format; }
        void file_format(const DataFile::FileType t) { format = t; }

        // tells if the acquisition is getting ready to start
        bool starting() const;

        // tells if the acquisition is running
        bool running() const;

        // returns true if run has to stop
        bool stop() const;

        // sets the stop condition
        void stop(bool x);

        // SAy if the DAQmanager is a GUI one
        bool is_gui() const { return _is_gui; }

        // Configuration
        void save_config(const char *);
        void load_config(const char *, bool load=true);

        /**
         * Static functions.
         * These are mainly used by the soap thread
         */
        static void set_the_manager(DAQmanager *m) { _mngr=m; }
        static DAQmanager *get_the_manager() { return _mngr; }
        static void set_daq_manager_data_file(const std::string &fnam);
        static void daq_manager_stop_run();
        static int  daq_manager_reset_board();
        static int  daq_manager_start_run(int run_type, int num_events, int sample_size, DataReceiver *D);
};

#endif /* DAQMANAGER_H_ */
