/*
 * DataRun.cc
 *
 *  Created on: May 12, 2010
 *      Author: lacasta
 */
#include <iostream>
#include <utils.h>
#include "DAQmanager.h"
#include "DataRun.h"
#include "DAQConfig.h"
#include "Analysis.h"
#include <ConfigFactory.h>

DataRun::DataRun(DAQmanager *D)
    : RunManager(D)
{

}

DataRun::~DataRun()
{
}

void DataRun::start_of_run()
{
    Alibava *alibava = daqm->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
    DataMonitor *mon = get_daq_manager()->get_monitor();
    ConfigFactory *cfg = ConfigFactory::instance();
    Analysis *ana = dynamic_cast<Analysis *>(cfg->get("analysis"));
    DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg->get("daq"));
    DAQmanager *daqm = get_daq_manager();

    ana->update_pedestals(true);
    if (mon)
        mon->create_signal_histogram();

    daqm->set_expected( daq->nevts() );
    daqm->set_received(0);

}

int DataRun::loop(std::ostream &oheader)
{
    int rc, nr;
    Alibava *alibava = daqm->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
    ConfigFactory *cfg = ConfigFactory::instance();
    DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg->get("daq"));
    int ievt, nevts = daq->nevts();
    oheader << nevts << "; " << daq->sample_size();
    for (ievt=0; ievt<nevts; ievt+=daq->sample_size())
    {
        if (get_daq_manager()->stop())
            break;

        check_new_event(ievt);

        rc = acquire(0);
        if (rc<0)
        {
            err_msg(_("Error %s while acquiring. See messages above.\n"), get_daq_manager()->get_board()->error_string(rc).c_str());
            break;
        }
        nr = read_data();
        if (nr<0 && rc!=Alibava::TimeOut)
        {
        	rc = nr;
            if ( rc != Alibava::DataSizeMismatch)
            {
                err_msg(_("Error: %s when reading data (ievt==%d)\n"), get_daq_manager()->get_board()->error_string(rc).c_str(), ievt);
            	break;
            }
        }

        get_daq_manager()->flush_data_file();
    }
    return rc;
}
