/*
 * LaserSyncRun.cc
 *
 *  Created on: May 12, 2010
 *      Author: lacasta
 */
#include <iostream>
#include "DAQmanager.h"
#include "LaserSyncRun.h"
#include "Analysis.h"
#include "ScanConfig.h"
#include "utils.h"
#include <ConfigFactory.h>

LaserSyncRun::LaserSyncRun(DAQmanager *b)
    : RunManager(b)
{

}

LaserSyncRun::~LaserSyncRun()
{
}

void LaserSyncRun::start_of_run()
{
    ConfigFactory *cfg = ConfigFactory::instance();
    LaserSyncConfig *lc = dynamic_cast<LaserSyncConfig *>(cfg->get("lasersync"));
    Analysis *ana = dynamic_cast<Analysis *>(cfg->get("analysis"));
    DAQmanager *daqm = get_daq_manager();
    DataFile *data_file = get_daq_manager()->out_file();

    ana->update_pedestals(false);

    DataMonitor *mon = get_daq_manager()->get_monitor();
    if (mon)
        mon->create_signal_profile("Delay",
                                   lc->npts(),
                                   lc->from(),
                                   lc->to() );
    if (data_file)
        data_file->scan_definition(0x10, lc->npts(), lc->from(), lc->to(), lc->nevts());

    daqm->set_expected(lc->npts()*lc->nevts());
    daqm->set_received(0);
}

int LaserSyncRun::loop(std::ostream &oheader)
{
    Alibava *alibava = get_daq_manager()->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
    unsigned int i, val, step;
    ConfigFactory *cfg = ConfigFactory::instance();
    LaserSyncConfig *lc = dynamic_cast<LaserSyncConfig *>(cfg->get("lasersync"));
    DAQmanager *daqm = get_daq_manager();
    DataFile *data_file = daqm->out_file();

    step = lc->step();
    oheader << lc->npts() << "; "
            << lc->from() << "; "
            << lc->to()  << "; "
            << lc->step();

    int rc = 0;
    set_value(true);
    for (i=0, val=lc->from(); i<lc->npts();i++,val+=step)
    {
        int start = daqm->get_received();
        if (get_daq_manager()->stop())
            break;

        check_new_event(i);

        daqm->get_board()->set_laser_delay(val);
        value(val);

        if ( (rc=acquire(val)) )
            break;

        if ( (rc=read_data())<0 )
            break;

        if (data_file)
        	data_file->new_point(start, daqm->get_received(), val);
    }
    set_value(false);
    return rc;
}

int LaserSyncRun::acquire(unsigned int val)
{
    Alibava *alibava = daqm->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
    ConfigFactory *cfg = ConfigFactory::instance();
    LaserSyncConfig *lc = dynamic_cast<LaserSyncConfig *>(cfg->get("lasersync"));

    return alibava->laser_acq(lc->nevts());
}
int LaserSyncRun::read_data()
{
    Alibava *alibava = daqm->get_board();
    ALIBAVA_TRACE_CLASS(alibava, "");
    return get_daq_manager()->get_board()->laser_reading(set_value(), value());

}
