/* -*- mode: c++ -*- */
#ifndef __ALIBAVA_SERVER_H__
#define __ALIBAVA_SERVER_H__
#include <glib.h>
#include <gio/gio.h>
#include <stdio.h>
#ifdef G_OS_UNIX
#include <sys/ioctl.h>
#endif

#define SERVER_PORT 50123

#ifdef ALIBAVA_SERVER
#ifndef ASCII
#define ASCII 1
#include <utils.h>
#undef ASCII
#else
#include <utils.h>
#endif
#else
#include <utils.h>
#endif

/*
 * The commands. This are defined like ioctls
 *
 */
#ifndef _IOC_NRBITS
#define _IOC_NRBITS     8
#define _IOC_TYPEBITS   8
#ifndef _IOC_SIZEBITS
# define _IOC_SIZEBITS  14
#endif

#ifndef _IOC_DIRBITS
# define _IOC_DIRBITS   2
#endif

#define _IOC_NRMASK     ((1 << _IOC_NRBITS)-1)
#define _IOC_TYPEMASK   ((1 << _IOC_TYPEBITS)-1)
#define _IOC_SIZEMASK   ((1 << _IOC_SIZEBITS)-1)
#define _IOC_DIRMASK    ((1 << _IOC_DIRBITS)-1)

#define _IOC_NRSHIFT    0
#define _IOC_TYPESHIFT  (_IOC_NRSHIFT+_IOC_NRBITS)
#define _IOC_SIZESHIFT  (_IOC_TYPESHIFT+_IOC_TYPEBITS)
#define _IOC_DIRSHIFT   (_IOC_SIZESHIFT+_IOC_SIZEBITS)
#ifndef _IOC_NONE
# define _IOC_NONE      0U
#endif

#ifndef _IOC_WRITE
# define _IOC_WRITE     2U
#endif

#ifndef _IOC_READ
# define _IOC_READ      1U
#endif

#endif

#define ALIBAVA_IOC_MAGIC  'P'

#define RESET_NR      0
#define DIE_NR        1
#define ERROR_NR      2
#define DO_READ_NR    3
#define ALIBAVA_MAX_COMMAND 3

#define RESET          _IO(ALIBAVA_IOC_MAGIC, RESET_NR)
#define DIE            _IO(ALIBAVA_IOC_MAGIC, DIE_NR)
#define ALIBAVA_ERROR          _IO(ALIBAVA_IOC_MAGIC, ERROR_NR)
#define DO_READ        _IOW(ALIBAVA_IOC_MAGIC, DO_READ_NR, size_t)

struct client_info
{
        int uid;
        GSocket *cfd;
        GSocket *fd;
        int sem;
        int command;
        int size;
        gint control;
        gint io;
        GIOChannel *control_chan;
        GIOChannel *io_chan;

        GMutex *mux;
};

#ifndef _IOC_DIR
/* used to decode ioctl numbers.. */
#define _IOC_DIR(nr)        (((nr) >> _IOC_DIRSHIFT) & _IOC_DIRMASK)
#define _IOC_TYPE(nr)       (((nr) >> _IOC_TYPESHIFT) & _IOC_TYPEMASK)
#define _IOC_NR(nr)     (((nr) >> _IOC_NRSHIFT) & _IOC_NRMASK)
#define _IOC_SIZE(nr)       (((nr) >> _IOC_SIZESHIFT) & _IOC_SIZEMASK)

#endif

#define SOCKET_NAME "/tmp/alibava.socket"
#define DAEMON_NAME "alibavad"
//#define DAEMON_PATH "/home/lacasta/workspace/alibava"

#ifndef ALIBAVA_SERVER
int do_open(const char *path, int flags);
int do_close(int fd);
int do_ioctl(int d, int req, ...);
int do_read(int d, void *, size_t);
int do_write(int d, void *, size_t);
void read_event(int fd);
#endif

#endif
