/* -*- mode: c++ -*- */
#ifndef __ATimer_included__
#define __ATimer_included__

#include <unistd.h>
#include <sys/time.h>

/**
 *  This class implements a timer.
 */
class ATimer
{
    private:
        /// start time of the timer
        struct timeval _start;
        /// stop time of the timer
        struct timeval _end;
        /// flags saying if the timer is active or not
        bool active;

    public:
        /// Constructor
        ATimer();
        /// Destructor
        ~ATimer() {}
        /// Starts the timer
        void start();
        /// Stops the timer
        void stop();
        /// resets the timer without stopping it
        void reset();
        /// returns the time in seconds elapsed since the start
        double operator()() const;
        /// returns true if the timer is active
        bool is_active() const
        {
            return active;
        }
};

inline ATimer::ATimer()
{
    start();
}
inline void ATimer::start()
{
    active = true;
    gettimeofday(&_start, NULL);
}
inline void ATimer::stop()
{
    active = false;
    gettimeofday(&_end, NULL);
}
inline void ATimer::reset()
{
    gettimeofday(&_start, NULL);
    _end = _start;
}
inline double ATimer::operator()() const
{
    double total;
    struct timeval elapsed;
    struct timeval end;

    if (active)
        gettimeofday(&end, NULL);
    else
        end = _end;
    if (_start.tv_usec > end.tv_usec)
    {
        end.tv_usec += 1000000;
        end.tv_sec--;
    }
    elapsed.tv_usec = end.tv_usec - _start.tv_usec;
    elapsed.tv_sec = end.tv_sec - _start.tv_sec;

    total = elapsed.tv_sec + ((double) elapsed.tv_usec) / 1.e6;
    return total;
}
#endif
