#!/usr/bin/env python
""" Test the soap interface
"""
import sys
import StringIO
from pylab import *
from alibavaSOAP import Alibava, Status
import socket
import SOAPpy


def main(host, port):
    """ Makes the tests
    """
    ion()
    server = Alibava(host, port)
    server.use_root=False
    #
    # Stop the run if there is any active
    #
    S = Status( server.getStatus() )
    if S.state:
        R = server.stopRun()
     
     
    nplots = 3
    figure(1) 
    #
    # We do a pedestal run
    #
    R = server.startPedestalRun(1000)
    # and we get the pedestal and noise values from the histograms
    pedestals = R[1]
    noise     = R[2]
    
    subplot(nplots,2,1)
    plot(r_[0:pedestals.nx], pedestals.buff)
    ylabel("Pedestals")
    subplot(nplots,2,2)
    plot(r_[0:pedestals.nx], noise.buff)
    ylabel("Noise")
    draw()
    
    #
    # Now a laser run
    #
    R = server.startLaserRun(1000)
    spectrum = R[0]
    subplot(nplots,1,2)
    step = (spectrum.xmax-spectrum.xmin)/spectrum.nx
    bins = r_[spectrum.xmin:spectrum.xmax:step]
    plot(bins, spectrum.buff, label="Laser")
    legend(["Laser Run"], loc=1)
    draw()

    #
    # Now we make a source run
    #
    R= server.startSourceRun(1000)
    spectrum = R[0]
    subplot(nplots,1,3)
    step = (spectrum.xmax-spectrum.xmin)/spectrum.nx
    bins = r_[spectrum.xmin:spectrum.xmax:step]
    plot(bins, spectrum.buff)
    legend(["RadSource Run"], loc=1)
    draw()
    
    
    figure(2)
    #
    # Get histogram pictures
    #
    hst = server.getHistogram("hstHitmap", "png")
    subplot(2,1,1)
    try:
        img = imread(hst)
        fig=imshow(img)
        fig.axes.get_xaxis().set_visible(False)
        fig.axes.get_yaxis().set_visible(False)
    except RuntimeError:
        pass
    draw()
    
    hst = server.getHistogram("hstCmmdNoise", "png")
    subplot(2,1,2)
    try:
        img = imread(hst)
        fig = imshow(img)
        fig.axes.get_xaxis().set_visible(False)
        fig.axes.get_yaxis().set_visible(False)
    except RuntimeError:
        pass
    draw()
    
    
    figure(3)
    # We do a calibration Run
    R = server.startCalibrationRun()
    subplot(1,1,1)
    hst = server.getHistogram("hstSignal", "png")
    try:
        img = imread(hst)
        fig = imshow(img)
        fig.axes.get_xaxis().set_visible(False)
        fig.axes.get_yaxis().set_visible(False)
    except RuntimeError:
        pass
    draw()    
    
    ioff()
    tight_layout()
    show()

if __name__ == "__main__":
    """
    """
    host = 'localhost'
    port =  10000
    
    
    try:
        inps = sys.argv[1]
        indx = inps.find(':')
        data = inps.split(':')
        nitem = len(data)
        if nitem == 2:
            if len(data[0]):
                host = data[0]
                
            if len(data[1]):
                try:
                    port = int(data[1])
                except ValueError:
                    print "Invalid port number", data[1]
                
        elif nitem == 1:
            if (len(data[0])):
                host = data[0]
                
    except IndexError:
        pass
        
    try:
        main(host, port)
    except socket.error as e:
        print "Could not connect", e
        print "Check that alibava-gui is running"
 